@@ -545,8 +545,8 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
             }
         }
 
-        // Fold + map op
-        "\\" => {
+        // Fold + map op, Cumulative fold + map op
+        "\\" | ":\\" => {
             let seq = Box::new(
                 parse_node(Rc::clone(&env), &right[0])
                     .literal_array()
@@ -572,7 +572,7 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
 
             if rest == Node::Variable("_".to_string()) {
                 Dynamic::from(res.as_ref().clone())
-            } else {
+            } else if op == "\\" {
                 let constructed = format!("{}", rest);
                 let seperator = constructed.trim().trim_matches('_');
                 let program = res
@@ -585,19 +585,44 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
                 // This looks like a Vec<Node>, but in reality it is a single Node (only one value)
                 let val = crate::build_ast(&program);
                 parse_node(Rc::clone(&env), &val[0])
+            } else {
+                let constructed = format!("{}", rest);
+                let seperator = constructed.trim().trim_matches('_');
+                let mut v = res.as_ref().clone();
+                for i in 1..v.len() {
+                    v[i] = parse_node(
+                        Rc::clone(&env),
+                        &crate::build_ast(&format!(
+                            "{} {} {}",
+                            v[i - 1].clone().into_node(),
+                            seperator,
+                            v[i].clone().into_node()
+                        ))[0],
+                    )
+                }
+
+                Dynamic::from(v)
             }
         }
 
         // Floor <right>
         ":v" => {
             let right = parse_node(Rc::clone(&env), &right[0]);
-            right.mutate_num(Num::floor)
+            if right.is_string() {
+                right.mutate_string(|s| s.to_ascii_lowercase())
+            } else {
+                right.mutate_num(Num::floor)
+            }
         }
 
         // Ceil <right>
         ":^" => {
             let right = parse_node(Rc::clone(&env), &right[0]);
-            right.mutate_num(Num::ceil)
+            if right.is_string() {
+                right.mutate_string(|s| s.to_ascii_uppercase())
+            } else {
+                right.mutate_num(Num::ceil)
+            }
         }
 
         // Inc <right>
@@ -733,6 +758,12 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
             ])
         }
 
+        // Evaluate <right> as arn code
+        "!." => {
+            let program = parse_node(Rc::clone(&env), &right[0]).literal_string();
+            parse_node(Rc::clone(&env), &crate::build_ast(&program)[0])
+        }
+
         // Zip <left> and <right>
         "z" => {
             let left = parse_node(Rc::clone(&env), &left[0])
@@ -899,10 +930,8 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
 
             if left.clone().literal_bool() {
                 left
-            } else if right.clone().literal_bool() {
-                right
             } else {
-                Dynamic::from(false)
+                right
             }
         }
 
@@ -1030,9 +1059,9 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
             parse_node_uniq(Rc::clone(&child_env), &right[0])
         }
 
-        // Count of entries in <r1> that, when bound by <r2>, yield a truthy value
+        // Count of entries in <r2> that, when bound by <r1>, yield a truthy value
         "/:" => {
-            let array = parse_node(Rc::clone(&env), &right[0])
+            let array = parse_node(Rc::clone(&env), &right[1])
                 .literal_array()
                 .set_env_self(Rc::clone(&env));
             let child_env = Rc::new(env.as_ref().clone());
@@ -1041,7 +1070,7 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
                 *FLOAT_PRECISION,
                 array
                     .filter(|val| {
-                        if let Node::Block(_, name) = &right[1] {
+                        if let Node::Block(_, name) = &right[0] {
                             child_env
                                 .borrow_mut()
                                 .define_var(name.as_ref().unwrap_or(&USCORE), val.clone())
@@ -1049,7 +1078,7 @@ pub fn parse_op(env: Env, op: &str, left: &[Node], right: &[Node]) -> Dynamic {
                             child_env.borrow_mut().define_var("_", val.clone());
                         }
 
-                        parse_node_uniq(Rc::clone(&child_env), &right[1]).literal_bool()
+                        parse_node_uniq(Rc::clone(&child_env), &right[0]).literal_bool()
                     })
                     .count(),
             ))
@@ -1201,27 +1230,97 @@ pub fn parse(ast: &[Node]) {
         "pi",
         Num::with_val(*FLOAT_PRECISION, rug::float::Constant::Pi),
     );
+    env.define_var(
+        "phi",
+        Num::with_val(
+            *FLOAT_PRECISION,
+            Num::parse("1.61803398874989484820458683436563811").unwrap(),
+        ),
+    );
     env.define_var("a", Vec::<Dynamic>::new());
     env.define_var("c", String::new());
     env.define_var("Fi", "Fizz".to_string());
     env.define_var("Bu", "Buzz".to_string());
+    let dummy_env = Rc::new(RefCell::new(Environment::init()));
+    env.define_var(
+        "sH",
+        Sequence::from_vec(
+            &crate::build_ast("1/2"),
+            crate::build_ast("/2")[0].clone(),
+            None,
+        )
+        .set_env_self(Rc::clone(&dummy_env)),
+    );
+    env.define_var(
+        "sA",
+        Sequence::from_vec(
+            &crate::build_ast("1"),
+            crate::build_ast("!")[0].clone(),
+            None,
+        )
+        .set_env_self(Rc::clone(&dummy_env)),
+    );
+    env.define_var(
+        "sE",
+        Sequence::from_vec(
+            &crate::build_ast("2"),
+            crate::build_ast("+2")[0].clone(),
+            None,
+        )
+        .set_env_self(Rc::clone(&dummy_env)),
+    );
+    env.define_var(
+        "sO",
+        Sequence::from_vec(
+            &crate::build_ast("1"),
+            crate::build_ast("+2")[0].clone(),
+            None,
+        )
+        .set_env_self(Rc::clone(&dummy_env)),
+    );
+    env.define_var(
+        "sF",
+        Sequence::from_vec(
+            &crate::build_ast("1 1"),
+            crate::build_ast("+")[0].clone(),
+            None,
+        )
+        .set_env_self(Rc::clone(&dummy_env)),
+    );
     // I don't care what people say, I am never adding a constant for "Hello, World!"
 
+    if MATCHES.is_present("input-left") {
+        let orig = env.get_var("_");
+        let as_vec = orig.literal_array().collect::<Vec<_>>();
+        env.define_var("_", as_vec[0].clone());
+        env.define_var("a", as_vec[1..].iter().cloned().collect::<Vec<_>>());
+    } else if MATCHES.is_present("input-right") {
+        let orig = env.get_var("_");
+        let as_vec = orig.literal_array().collect::<Vec<_>>();
+        env.define_var("_", as_vec[1..].iter().cloned().collect::<Vec<_>>());
+        env.define_var("a", as_vec[0].clone());
+    }
+
     // Defined functions
-    env.define(["o", "out"], |_, d| {
+    env.define(["ol", "outl"], |_, d| {
         println!("{}", d);
         d
     });
+    env.define(["o", "out"], |_, d| {
+        print!("{}", d);
+        d
+    });
     def_builtins! {env;
-        "f", "fact":        r#"*\(~||[1])"#;
+        "f", "fact":        r#"~||[1]&*\"#;
         "me", "mean":       r#"(+\)/(#"#;
-        "ma", "max":        r#"(:>) :{"#;
+        "ma", "max":        r#":>&:{"#;
         "mo", "mode":       r#":@&ma:{"#;
-        "mi", "min":        r#"(:<) :{"#;
+        "mi", "min":        r#":<&:{"#;
         "med", "median":    r#"(:-#&%2=0)&&:-(((:<)?(--:-#))+((:<)?:-#))||(:<)?:v:-#"#;
         "sdev":             r#":/((@v{:*(v-me)).me"#;
         "crt", "cartesian": r#":{@a{:}@a<>}&:_"#;
-        "eq", "equal":      r#":@#=1"#
+        "eq", "equal":      r#":@#=1"#;
+        "pst", "powerset":  r#":<(0->2^(#);2@a{|{+0&&[:}]||[}\a.<z"#
     };
 
     let env: Env = Rc::new(RefCell::new(env));
