@@ -602,6 +602,21 @@ where
     }
 }
 
+impl From<Sequence> for Dynamic {
+    fn from(v: Sequence) -> Self {
+        Self {
+            val: Val::Array(Box::new(v)),
+            cur: 4,
+        }
+    }
+}
+
+impl From<Node> for Dynamic {
+    fn from(v: Node) -> Self {
+        crate::parser::parse_node(Rc::new(RefCell::new(Environment::init())), &v)
+    }
+}
+
 #[allow(clippy::from_over_into)]
 impl Into<Node> for Dynamic {
     fn into(self) -> Node {
@@ -686,7 +701,7 @@ impl Sequence {
 
     #[inline]
     pub fn set_env(&mut self, env: Env) {
-        self.env = Some(env);
+        self.env = Some(Rc::new(env.as_ref().clone()));
     }
 
     #[inline]
@@ -716,6 +731,11 @@ impl Sequence {
         } else {
             self.index += 1;
             let block = self.traverse_replace(self.block.clone());
+            self.env
+                .as_ref()
+                .unwrap()
+                .borrow_mut()
+                .define_var("p", Dynamic::from(self.cstr.as_slice()));
             let res = crate::parser::parse_node(Rc::clone(self.env.as_ref().unwrap()), &block);
             self.cstr.push(res.clone());
 
